package net.jxta.impl.shell.bin.example5_1;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import net.jxta.impl.protocol.ResolverResponse;

import net.jxta.protocol.ResolverQueryMsg;
import net.jxta.protocol.ResolverResponseMsg;

import net.jxta.resolver.QueryHandler;
import net.jxta.resolver.ResolverService;


/**
 * A simple handler to process Resolver queries.
 */
class ExampleHandler implements QueryHandler
{
    /**
     * The ResolverService used to handle queries and send responses.
     */
    private ResolverService resolver;


    /**
     * Create a new handler using the given ResolverService instance.
     *
     * @param   aResolver the ResolverService instance to use to 
     *          handle queries and send responses.
     */
    public ExampleHandler(ResolverService aResolver)
    {
        this.resolver = aResolver;
    }

    /**
     * Processes the Resolver query message and returns a response. 
     *
     * @param       query the Resolver Query Message to be processed.
     * @return      an integer representing the success of processing,
     *              or need to repropagate the query.
     */
    public int processQuery(ResolverQueryMsg query)
    {
        ResolverResponse response;
        ExampleQueryMsg eq;
        double answer = 0.0;
        
        System.out.println("Processing query...");

        // Parse the message from the query string.
        try
        {
            eq = new ExampleQueryMsg(
                new ByteArrayInputStream((query.getQuery()).getBytes()));

            // Perform the calculation.
            answer = Math.pow(eq.getBase(), eq.getPower());
        
            // Create the response message.
            ExampleResponseMsg er = new ExampleResponseMsg(eq.getBase(),
                eq.getPower(), answer);

            // Wrap the response message in a resolver response message.
            response = new ResolverResponse("ExampleHandler",
                null, query.getQueryId(), er.toString());

            // Send the response using the resolver.
            resolver.sendResponse(query.getSrc(), response);

            // Signal that the resolver handled sending the response.
            return resolver.OK;
        }
        catch (IOException e)
        {
            // Signal that the query should be repropagated.
            return resolver.Repropagate;
        }
    }

    /**
     * Process a Resolver response message.
     *
     * @param   response the Resolver Response Message to be processed.
     */
    public void processResponse(ResolverResponseMsg response)
    {
        ExampleResponseMsg er;
        
        System.out.println("Processing response...");

        try
        {
            // Extract the message from the Resolver response.
            er = new ExampleResponseMsg(
                new ByteArrayInputStream(
                    (response.getResponse()).getBytes()));
            
            // Print out the answer given in the response.
            System.out.println(
                "The value of " + er.getBase() + " raised to "
                    + er.getPower() + " is: " + er.getAnswer()); 
        }
        catch (IOException e)
        {
            // This is not the right type of response message, or
            // the message is improperly formed. Ignore the message,
            // do nothing.
        }
    }
}